@@ -2,12 +2,11 @@
 #
 # Inline::Octave - 
 #
-# $Id: Octave.pm,v 1.13 2002/01/19 01:08:35 aadler Exp $
+# $Id: Octave.pm,v 1.17 2003/01/07 02:47:49 aadler Exp $
 
 package Inline::Octave;
 
-
-$VERSION = '0.15';
+$VERSION = '0.17';
 require Inline;
 @ISA = qw(Inline);
 use Carp;
@@ -17,7 +16,7 @@ use vars qw( $octave_object );
 # set values which should change to this,
 # if it doesn't change, we have an error.
 my $retcode_string= "[-101101.101101,-918273.6455,-178.9245867]";
-my $retcode_value=   [-101101.101101,-918273.6455,-178.9245867];
+my $retcode_value=   eval $retcode_string;
 
 sub register {
 #  print "REGISTERING\n";
@@ -341,6 +340,46 @@ END {
    Inline::Octave::stop_interpreter() if $octave_object;
 }
 
+# define a generic Inline::Octave->new
+# which will figure out which class best suits the
+# new object
+# This is not OO. It directly calls the other packages,
+# but the idea is to put all the non OO stuff here.
+
+sub fix_octave_type
+{
+    my $m = shift;
+    if    ($m->{"complex"}) {
+        return Inline::Octave::ComplexMatrix->new($m);
+    }
+    elsif ($m->{"string"}) {
+        return Inline::Octave::String->new($m);
+    }
+    else {
+        return Inline::Octave::Matrix->new($m);
+    }
+}
+sub new
+{
+   my $class = shift; #this is Inline::Octave, but we don't care
+   my ($m, $rows, $cols) = @_;
+
+   if ( (ref($m) eq "Inline::Octave::Matrix"       ) or
+        (ref($m) eq "Inline::Octave::String"       ) or 
+        (ref($m) eq "Inline::Octave::ComplexMatrix") ) {
+      return fix_octave_type($m);
+   }
+   elsif ((ref $m eq "ARRAY") ||
+          (ref $m eq "Math::Complex" ) ||
+          (ref $m eq "" ) ) {
+          # here we have data or numbers, we write it
+          # to a complex matrix just in case
+      return fix_octave_type( Inline::Octave::ComplexMatrix->new($m) );
+   } else {
+      croak "Can't construct Inline::Octave from Perl var of type:".ref($m);
+   }
+} #new - no return required at end
+
 #
 # Inline::Octave::String contains the code
 # to handle octave String objects
@@ -401,6 +440,33 @@ sub new
    return $self;
 }   
 
+package Inline::Octave::ComplexMatrix;
+use Carp;
+use Math::Complex;
+@ISA= qw(Inline::Octave::Matrix);
+
+sub write_out_matrix {
+   my $self=  shift;
+   my $rows=  shift;
+   my $cols=  shift;
+   my $vals=  shift;
+   my $trans= shift;
+   my $vname= shift;
+
+   my (@real, @imag);
+   for (@$vals) {
+      push @real, Re( $_ );
+      push @imag, Im( $_ );
+   }
+
+   my $code= "$vname=fread(stdin,[$rows,$cols],'double')$trans +".
+                    "fread(stdin,[$rows,$cols],'double')$trans*I;\n".
+             pack( "d".($rows*$cols*2) , ( @real, @imag)  );
+   return $code;
+}
+
+
+
 #
 # Inline::Octave::Matrix contains the code
 # to handle octave Matrix objects
@@ -411,6 +477,7 @@ sub new
 # new IOM( [1,2,3,4], 2, 2) -> Matrix, rows, cols
 package Inline::Octave::Matrix;
 use Carp;
+use Math::Complex;
 
 $varcounter= 10000001;
 
@@ -428,11 +495,9 @@ sub new
    my $do_transpose= '';
    my $code;
 
-   if    (ref $m      eq "Inline::Octave::Matrix") {
-      my $prev_varname= $m->{varname};
-      $code= "$varname= $prev_varname;";
-   }
-   elsif (ref $m      eq "Inline::Octave::String") {
+   if  ( (ref $m      eq "Inline::Octave::Matrix") ||
+         (ref $m      eq "Inline::Octave::String") ||
+         (ref $m      eq "Inline::Octave::ComplexMatrix") ) {
       my $prev_varname= $m->{varname};
       $code= "$varname= $prev_varname;";
    }
@@ -456,7 +521,10 @@ sub new
       $cols= 1 unless defined $cols;
       @vals= @{$m};
    }
-   elsif (ref $m eq "" ) {
+   elsif ((ref $m eq "Math::Complex") ||
+          (ref $m eq "" )) {
+          # here we have data or numbers, we write it
+          # to a complex matrix just in case
       $rows= 1 unless defined $rows;
       $cols= 1 unless defined $cols;
       @vals = ($m);
@@ -471,8 +539,7 @@ sub new
       croak "Matrix is not size ${cols}x${rows}" unless
              @vals == $rows*$cols ;
 
-      $code= "$varname=fread(stdin,[$rows,$cols],'double')$do_transpose;\n".
-             pack( "d".($rows*$cols) , @vals );
+      $code= $self->write_out_matrix( $rows,$cols,\@vals, $do_transpose, $varname );
    }
    Inline::Octave::interpret(0, $code );
 
@@ -481,6 +548,19 @@ sub new
    return $self;
 }
 
+sub write_out_matrix {
+   my $self=  shift;
+   my $rows=  shift;
+   my $cols=  shift;
+   my $vals=  shift;
+   my $trans= shift;
+   my $vname= shift;
+
+   my $code= "$vname=fread(stdin,[$rows,$cols],'double')$trans;\n".
+             pack( "d".($rows*$cols) , @$vals );
+   return $code;
+}
+
 sub rows
 {
    my $self= shift;
@@ -522,27 +602,46 @@ sub store_size
    $self->{string}= $4;
 }             
 
+sub read_back_matrix {
+   my $self   = shift;
+   my $extra  = shift || "";
+   my $varname= $self->name;
+   my @list;
+   if ( $self->{complex} ) {
+      my $code = "fwrite(stdout, real( $varname$extra ),'double');".
+                 "fwrite(stdout, imag( $varname$extra ),'double');";
+      my $retval= Inline::Octave::interpret(0, $code );
+      my $half  = length( $retval ) /2;
+      my $size= $self->elements();
+      my @real= unpack "d$size", substr( $retval,     0, $half);
+      my @imag= unpack "d$size", substr( $retval, $half, $half);
+      for (my $i=0; $i< $size ; $i++ ) {
+         if ($imag[$i] == 0 ) {
+            push @list, $real[$i];
+         } else {
+            push @list, cplx( $real[$i], $imag[$i] );
+         }
+      }
+   } else { # not complex
+      my $code = "fwrite(stdout, $varname$extra,'double');";
+      my $retval= Inline::Octave::interpret(0, $code );
+      my $size= $self->elements();
+      @list= unpack "d$size", $retval;
+   };
+   return \@list;
+}
+
 sub as_list
 {
    my $self = shift;
-   my $varname= $self->name;
-   croak "Can't handle complex" if $self->{complex};
-   my $code = "fwrite(stdout, $varname,'double');";
-   my $retval= Inline::Octave::interpret(0, $code );
-   my $size= $self->elements();
-   my @list= unpack "d$size", $retval;
-   return @list;
+   my $list = $self->read_back_matrix();
+   return @$list;
 }
 
 sub as_matrix
 {
    my $self = shift;
-   my $varname= $self->name;
-   croak "Can't handle complex" if $self->{complex};
-   my $code = "fwrite(stdout, $varname','double');"; # use transpose
-   my $retval= Inline::Octave::interpret(0, $code );
-   my $size= $self->elements();
-   my @list= unpack "d$size", $retval;
+   my @list= @{ $self->read_back_matrix("'") };
    my @m;
    my $cols= $self->cols();
    my $rows= $self->rows();
@@ -562,12 +661,7 @@ sub sub_matrix
    my $col_specv = new Inline::Octave::Matrix( shift );
    my $col_specn = $col_specv->name;
 
-   my $varname= $self->name;
-   croak "Can't handle complex" if $self->{complex};
-   my $code = "fwrite(stdout, $varname($row_specn,$col_specn)','double');";
-   my $retval= Inline::Octave::interpret(0, $code );
-   my $size= $self->elements();
-   my @list= unpack "d$size", $retval;
+   my @list= @{ $self->read_back_matrix("($row_specn,$col_specn)'") };
    my @m;
    my $cols= $col_specv->max_dim();
    my $rows= $row_specv->max_dim();
@@ -606,7 +700,9 @@ sub disp
    my $self = shift;
    my $varname= $self->name;
    my $code = "disp( $varname );";
-   return Inline::Octave::interpret(0, $code );
+   my $retval= Inline::Octave::interpret(0, $code );
+   chomp ($retval);
+   return $retval;
 }   
 
 sub name
@@ -628,16 +724,16 @@ use overload
 
 sub oct_matrix_arithmetic
 {
-   my $a= new Inline::Octave::Matrix( shift );
-   my $b= new Inline::Octave::Matrix( shift );
+   my $a= Inline::Octave->new( shift );
+   my $b= Inline::Octave->new( shift );
    ($b,$a)= ($a,$b) if shift;
    my $op= shift;
 
-   my $v= new Inline::Octave::Matrix( $retcode_value );
+   my $v= Inline::Octave::Matrix->new( $retcode_value );
 
    my $code= $v->name."=". $a->name ." $op ". $b->name .';';
    run_math_code( $code, $v);
-   return $v;
+   return Inline::Octave->new($v);
 }   
 
 # usage
@@ -701,7 +797,12 @@ sub replace_matrix
 #
 # create methods for the various math functions
 #
-{
+
+# include guard
+$Inline::Octave::methods_defined=0;
+unless ($Inline::Octave::methods_defined) {
+   $Inline::Octave::methods_defined=0;
+
    my %methods= (
       abs           => 1, acos          => 1, acosh         => 1,
       all           => 1, angle         => 1, any           => 1,
@@ -729,8 +830,9 @@ sub replace_matrix
       tanh          => 1, zeros         => 1,
    );
 
-   for my $meth ( keys %methods ) {
+   for my $meth ( sort keys %methods ) {
       no strict 'refs';
+      no warnings 'redefine';
       my $nargout= $methods{$meth};
       *$meth = sub {
          my $code= "[";
@@ -813,8 +915,23 @@ TODO LIST:
        Inline::Octave::interpret(0, $code );
    9. Add support for passing Strings to Octave
   10. Errors in Octave should die in perl
+       - this involves a tricky Open3 read of stderr.
+  11. Refactor out the common code in Inline::Octave::Matrix
+       into an Inline::Octave::Variable class
 
 $Log: Octave.pm,v $
+Revision 1.17  2003/01/07 02:47:49  aadler
+mods to remove warnings from tests
+
+Revision 1.16  2002/10/29 03:59:29  aadler
+improved complex code
+
+Revision 1.15  2002/05/01 02:19:40  aadler
+Initial work to add Inline::Octave::ComplexMatrix
+
+Revision 1.14  2002/03/08 21:15:34  aadler
+Mods to add sub_matrix, and OO type cleanups
+
 Revision 1.13  2002/01/19 01:08:35  aadler
 new docs and new matrix methods
 
@@ -903,6 +1020,9 @@ fread(stdin, [dimx dimy], "double"), and read similarly.
 Inline::Octave::Matrix variables in perl are tied to the octave
 variable. When a destructor is called, it sends a "clear varname"
 command to octave.
+
+Additionally, there are Inline::Octave::ComplexMatrix and 
+Inline::Octave::String types for the corresponding variables.
                    
 I initially tried to bind the C++ and liboctave to perl, but
 it started to get really hard - so I took this route.
@@ -1179,6 +1299,20 @@ Then these methods will make life more convenient.
 
    $a->replace_matrix( [1,4], [1,4], [ [8,7],[6,5] ] );
 
+=head1 Using Inline::Octave::ComplexMatrix
+
+Inline::Octave::ComplexMatrix should work very similarly
+to Inline::Octave::Matrix's.  The perl Math::Complex type
+is used to map octave complex numbers.
+
+Note, however, that the Math::Complex type in perl is heavy
+- it takes lots of memory and time compared to the native
+implementation in Octave.
+
+   use Math::Complex;
+   my $x= Inline::Octave::ComplexMatrix->new([1,1,2,3 + 6*i,4]);
+   print $x->disp();
+
 =head1 Using Inline::Octave::String
 
 Inline::Octave::String is a subclass of Inline::Octave::Matrix
@@ -1212,6 +1346,12 @@ possible to run faster than in each. (ie using octave
 for matrix operations, and running loops and text
 stuff in perl)
 
+One performance issue is Complex matrix math in perl. 
+The perl Math::Complex type is quite heavy, and for 
+large matrices this work is done for each element.
+You should try to do the complex stuff in octave,
+and only pull back small matrices into perl.
+
 
 =head1 AUTHOR
 
